
function apia_event(form) {

  // APXML event injection has been converted to a JavaScript object
  // that can be serialized and sent to xMatters.
  
  if (IALOG.isDebugEnabled()) {
    IALOG.debug("{0}{1}: JSON received {2}", loggingHeader, form.properties["aid"], JSON.stringify(form, null, 2));
  }
  
  if (!form["conferences"] && form.properties["clearable"] != null && "false".equalsIgnoreCase(form.properties["clearable"])) {
      // set responses
      form.responses = RESPONSE_OPTIONS_WHEN_NOT_CLEARABLE;
  }
  
  if (form.properties["person_or_group_id"]) {
    var recipient = form.properties["person_or_group_id"];
    form.recipients = [{'targetName' : recipient }];
    delete form.properties["person_or_group_id"];

    IALOG.debug("{0}{1}: recipient set to {2}", loggingHeader, form.properties["aid"], recipient);
  }

  if ( form.properties["deleteclearedevent"] ) {
    delete form.properties["deleteclearedevent"];
  }
  
  return form;
}

// Overrides event.js implementation to support multiple forms

function apia_input(apxml) {
  // Inject event
  var aid = apxml.getValue("aid");
  var mhandle = apxml.getValue("mhandle");
  var incidentId = apxml.getValue("incident_id");
  var cause = apxml.getValue("cause");

  if (cause != null) {
    cause = "0x" + ("00000000" + cause).substr(-8);
  }
  
  var deleteClearedEvent = "true" === String(apxml.getValue("deleteClearedEvent")).toLowerCase();
  
  IALOG.info("{0}{1}: Received request to inject event", loggingHeader, aid);    
  
  IALOG.debug("{0}{1}: URL={2}, INITIATOR={3}, PASSWORD={4}", loggingHeader, aid, XMATTERS_REB_FORM_CASPECTRUM, INITIATOR, PASSWORD); 
  var response = XMRESTAPI.submitApxml( XMATTERS_REB_FORM_CASPECTRUM, apxml, deleteClearedEvent ? getExistingEventsFilter(incidentId) : null, null );
  
  if (response) {
    // if request was actually sent
    if (response.status == XMRESTAPI.RESPONSE_SUCCESS) {
      var recipients = apxml.getValue("person_or_group_id");
      IALOG.info("{0}{1}: Event {2} was sent to {3}", loggingHeader, aid, JSON.parse(response.body).id, recipients); 
  	  // process the event
      var sentMessage = "[Alarm " + aid + "][" + cause + "] Alarm " + aid + " sent to xMatters";
      var update_params = " " + mhandle + " " + incidentId + " " + AlarmStatus + " \"" + annotationPrefix + sentMessage + "\"";
	    initvariables();
	    logNotification(sentMessage);
	    executeSpectrumCli(cmdPath, update_cmd + update_params, null);      
    } else {
      IALOG.warn("{0}: FAILURE ({1})", incidentId, response.status);      
      XMRESTAPI.checkResponse( response );
    }
  }
  
  // Create conferences
  var sev = new String(apxml.getValue("sev"));
  
  if (XMATTERS_ENABLE_CONFERENCES && (sev == "Critical" || sev == "Major")) {  
    IALOG.debug("{0}{1}: Creating conference request", loggingHeader, aid);
    apxml.setToken("xmconferences", JSON.stringify([{ 'name': null }]));

    IALOG.debug("{0}{1}: URL={2}, INITIATOR={3}, PASSWORD={4}", loggingHeader, aid, XMATTERS_REB_FORM_CASPECTRUM_CONFERENCE, INITIATOR, PASSWORD); 
    var response = XMRESTAPI.submitApxml( XMATTERS_REB_FORM_CASPECTRUM_CONFERENCE, apxml, null, null, false );
    
    if (response) {
      // if request was actually sent
      if (response.status == XMRESTAPI.RESPONSE_SUCCESS) {
        var recipients = apxml.getValue("person_or_group_id");
        IALOG.info("{0}{1}: Conference {2} was created for {3}", loggingHeader, aid, JSON.parse(response.body).id, recipients); 
        
        // process the conference
        var sentMessage = "[Alarm "+aid+"]["+cause+"] Conference for alarm " + aid + " was created in xMatters";
        var update_params = " " + mhandle + " " + incidentId + " " + AlarmStatus + " \"" + annotationPrefix + sentMessage + "\"";
        initvariables();
        logNotification(sentMessage);
        executeSpectrumCli(cmdPath, update_cmd + update_params, null);      

      } else {
        IALOG.warn("{0}: FAILURE ({1})", incidentId, response.status);      
        XMRESTAPI.checkResponse( response );
      }
    }    
  }
}

/**
 * ---------------------------------------------------------------------------------------------------------------------
 * getExistingEventsFilter
 * @param incidentId ID of the incident ticket
 *
 * return a Javascript object that contains the event filter parameters 
 * ---------------------------------------------------------------------------------------------------------------------
 */
function getExistingEventsFilter(incidentId) {
  return { status : "ACTIVE", properties : { 'incident_id#en' : incidentId } } 
}

