// =====================================================================================================================
// caspectrum.js
//
// Main file for the CA Spectrum integration service. Handles communication between CA Spectrum and xMatters.
//
// Copyright 2012-2015 xMatters
// =====================================================================================================================
importPackage(java.lang);
importPackage(java.io);

importPackage(Packages.com.alarmpoint.spectrum.cli);

/*
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLMessage);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLToken);
importClass(Packages.com.alarmpoint.integrationagent.util.EventDeduplicator);
importClass(Packages.com.thoughtworks.xstream.XStream);
importClass(Packages.com.thoughtworks.xstream.converters.reflection.PureJavaReflectionProvider);
*/
importClass(Packages.org.apache.log4j.Logger);
/*
var xStream = new XStream(new PureJavaReflectionProvider())
*/
// declare the CA logger prior to creating the default js logger
var caLogger = Logger.getLogger("com.alarmpoint.notification.delivery");

var INT_VERSION = "caspectrum-2-0";

load("integrationservices/" + INT_VERSION + "/configuration.js");

// REST API support 
load("lib/integrationservices/javascript/log.js");
load("lib/integrationservices/javascript/xmutil.js");
load("lib/integrationservices/javascript/apxml.js");
load("lib/integrationservices/javascript/xmio.js");

load("integrationservices/" + INT_VERSION + "/xmrestapi.js");

// Integration files
load("integrationservices/" + INT_VERSION + "/caspectrum-event.js");
load("integrationservices/" + INT_VERSION + "/caspectrum-callbacks.js");

// Location of SPECTRUM AlarmNotifier
var cmdPath;
var showPath;

/**
 * handles annotations sent from the Callout APS script
 */
function handleCalloutAnnotate( incidentId, message )
{ 
  logNotification("[Alarm " + incidentId + "]" + message);
}

/**
 * This method sets the variables representing the CLI commands to be run based
 * on operating system.
 */
function initvariables()
{
  var os = System.getProperty("os.name");
  if (os.toLowerCase().startsWith("win"))
  {
    cmdPath  = CAS_HOME + "\\Notifier";
    showPath = CAS_HOME + "\\vnmsh";
    IALOG.debug("{0}Setting Windows type path", loggingHeader);
  }
  else
  {
    cmdPath  = CAS_HOME;
    showPath = CAS_HOME + "/vnmsh";
    IALOG.debug("{0}Setting Unix type path", loggingHeader);
  }
}

function apia_webservices_init() {
  initvariables();
}

/**
 * Run the Spectrum CLI command and wait for its response.
 * Read the response and log it. 
 **/ 
function executeSpectrumCli(path, command, type) {
  IALOG.info("{0}About the run cli command {1}", loggingHeader, command);
  
  if (DEBUG) return "DEBUG";
  
  // Execute the program using path as current directory
  var cliMgr = new SpectrumCLIManager(path);
  IALOG.debug("{0}cli - {1}", loggingHeader, command);
  var response = cliMgr.querySpectrum(command, type);
  IALOG.debug("{0}response - {1}", loggingHeader, response);

  return response;
}

/**
 * Called from the web application to run a show command
 */
function show(params)
{
  initvariables();
  executeSpectrumCli(showPath, "connect", null);
  var response = executeSpectrumCli(showPath, params.remove("cli_action"), params.remove("cli_type"));
  executeSpectrumCli(showPath, "disconnect", null);

  return response;
}

// -----------------------------------------------------------------------------
// Utility functions
// -----------------------------------------------------------------------------

/**
 * getIncidentID
 *
 * Check whether additionalTokens and incident_id are present, and return the content if they are. 
 * If not, an exception is thrown.
 *
 * @param msg
 */
function getIncidentID( msg ) { 
  return getProperty( msg, "incident_id" );
}

/**
 * ---------------------------------------------------------------------------------------------------------------------
 * getProperty
 *
 * Check whether additionalTokens and property with propertyName name are present, and return the content if they are. 
 * If not, an exception is thrown.
 *
 * @param msg
 * @param propertyName
 * ---------------------------------------------------------------------------------------------------------------------
 */
function getProperty( msg, propertyName, defaultValue ) {
  var hasID = ('additionalTokens' in msg) && (propertyName in msg.additionalTokens);
  
  if ( !hasID ) {
    if (typeof defaultValue !== "undefined") {
      return defaultValue;
    } else {
      throw { name: "DataException", message: propertyName + " property is not found in event callback data of type " + msg.xmatters_callback_type + " for event ID " + msg.eventidentifier 
        + ". Make sure 'Include in Callbacks' is set for " + propertyName + " property in the form's layout"};
    }
  }
  
  return msg.additionalTokens[propertyName];
}

/**
 * This function was created to log all AlarmPoint notification deliveries.
 * The intent is that the log defined by "com.alarmpoint.notification.delivery" would fulfill the role of an audit log.
 */  
function logNotification(msg)
{
  caLogger.info(msg);
}
