XMRESTAPI = {};

(function(){

  XMRESTAPI.RESPONSE_SUCCESS = "200";
  XMRESTAPI.RESPONSE_DATA_VALIDATION_ERROR = "400";

  // private functions
  
	function getEventFilterAsString(filter) {	
    var text = "";
    
    for (var property in filter) {
      var value = filter[property];
      
      if (property == "properties") {
        var properties = "";
        for (var p in value) {
          if (properties != "") properties += ",";
          properties += p + "=" + String(value[p]);
        }
        value = encodeURIComponent(properties);
      } else {
        value = String(value);
      }
      
      if (value != null) {
        if (text != "") text += "&";
        text += property + "=" + value;
      }
    }
    
    return text;
	}
  
  // public functions
  
	XMRESTAPI.getEvents = function(formURL, eventFilter) {
    IALOG.debug("XM REST API: getEvents for " + eventFilter);
    var parameters = getEventFilterAsString(eventFilter);
    
    var url = formURL.substring(0, formURL.indexOf("forms")) + "events?" + parameters;
    
    var response = XMIO.get(url);
    IALOG.debug("XM REST API: getEvents received " + response.status + " " + response.body);    
    
    XMRESTAPI.checkResponse( response );      
    
    return JSON.parse(response.body);
  };

	XMRESTAPI.setEventStatus = function(formURL, eventFilter, status) {
    IALOG.debug("XM REST API: setEventStatus for " + eventFilter + " to " + status);
    var count = 0;
    
    var events = XMRESTAPI.getEvents( formURL, eventFilter );
    
    for (var i = 0; i < events.total; i++) {
      var event = events.records[i];
      IALOG.debug("XM REST API: setEventStatus for {0} to {1}", event.href, status); // href looks like /reapi/2013-12-01/events/<id>
      
      var url = formURL.substring(0, formURL.indexOf("/reapi")) + event.href;
      
      var response = XMIO.put(JSON.stringify({ 'status': status }), url);
      IALOG.debug("XM REST API: setEventStatus received {0} {1}", response.status, response.body);

      XMRESTAPI.checkResponse( response, {status : 409} ); // ignore conflict errors      
      
      count++;      
    }

    IALOG.info("XM REST API: setEventStatus events {0}: {1}", status, count);    
    return count;
  };

	XMRESTAPI.deleteEvents = function(formURL, eventFilter) {
    IALOG.debug("XM REST API: deleteEvents for {0}", eventFilter);
    return XMRESTAPI.setEventStatus(formURL, eventFilter, "TERMINATED");
  };
  
  // Submit Apxml
  XMRESTAPI.submitApxml = function(formURL, apxml, existingEventsFilter, keys, deduplicationFilter) {

  	if (deduplicationFilter !== false && XMUtil.deduplicate(apxml)) {
      return;
    }
    
    if ( existingEventsFilter ) {
      XMRESTAPI.deleteEvents( formURL, existingEventsFilter );
    }
    
  	var eventObj = XMUtil.createEventTemplate();
    var newKeys = keys ? keys : null;
    var apxmlAsObj = APXML.toEventJs(apxml, eventObj, newKeys);
    
    IALOG.info("Calling JavaScript method apia_event");
    var obj = apia_event( apxmlAsObj );
    var json = JSON.stringify(obj);    
    
    IALOG.info("XM REST API: Post to {0} {1}", formURL, json);
    return XMIO.post(json, formURL);
  }

  // Utility methods  
  XMRESTAPI.checkResponse = function(response, ignoreError) {
  
    if ( response.status !== undefined && response.status != XMRESTAPI.RESPONSE_SUCCESS) {
      // Ignore status?
      if (ignoreError && ignoreError['status'] != null && ignoreError['status'] == response.status) {
        IALOG.debug("XM REST API: checkResponse status {0} will be treated as success", response.status);
        return response;
      }
      var error; 
      try {
        var body = JSON.parse(response.body);
        
        if (ignoreError && ignoreError['type'] != null && ignoreError['type'] == body.type) {
          IALOG.debug("XM REST API: checkResponse error type {0} will be treated as success", response.status);
          return response;
        }
        error = body.message;
      } catch (e) {
        error = "xMatters server returned status " + response.status;
      }     
      throw error;
    }
    
    return response;
  };
  
  XMRESTAPI.getFormURL = function(webServiceURL, formUID) {
    
    if (formUID.startsWith("http"))
      return formUID;
      
    var triggers = webServiceURL.indexOf("/triggers");
    if (triggers >= 0) {
      // 'form' parameter is treated as form UID e.g. https://<xM server>/reapi/2015-01-01/forms/<formUID>/triggers
      return webServiceURL.substring(0, webServiceURL.lastIndexOf('/', triggers-1)) + "/" + formUID + "/triggers";
    }

    IALOG.warn("XM REST API:: Unrecognized WEB_SERVICE_URL format. getFormURL will use {0} 'as is'", webServiceURL);
    
    return webServiceURL;
  };
})();